@@ -248,6 +248,14 @@ void do_show_super_stats(int argc, char *argv[])
 	close_pager(out);
 }
 
+void do_dirty_filesys(int argc, char *argv[])
+{
+	if (check_fs_open(argv[0]))
+		return;
+	
+	ext2fs_mark_super_dirty(current_fs);
+}
+
 struct list_blocks_struct {
 	FILE	*f;
 	int	total;
@@ -565,7 +573,7 @@ static void modify_u8(char *com, const char *prompt,
 		      const char *format, __u8 *val)
 {
 	char buf[200];
-	u_char v;
+	unsigned long v;
 	char *tmp;
 
 	sprintf(buf, format, *val);
@@ -575,7 +583,7 @@ static void modify_u8(char *com, const char *prompt,
 		buf[strlen (buf) - 1] = '\0';
 	if (!buf[0])
 		return;
-	v = strtol(buf, &tmp, 0);
+	v = strtoul(buf, &tmp, 0);
 	if (*tmp)
 		com_err(com, 0, "Bad value - %s", buf);
 	else
@@ -586,7 +594,7 @@ static void modify_u16(char *com, const char *prompt,
 		       const char *format, __u16 *val)
 {
 	char buf[200];
-	u_short v;
+	unsigned long v;
 	char *tmp;
 
 	sprintf(buf, format, *val);
@@ -596,7 +604,7 @@ static void modify_u16(char *com, const char *prompt,
 		buf[strlen (buf) - 1] = '\0';
 	if (!buf[0])
 		return;
-	v = strtol(buf, &tmp, 0);
+	v = strtoul(buf, &tmp, 0);
 	if (*tmp)
 		com_err(com, 0, "Bad value - %s", buf);
 	else
@@ -607,7 +615,7 @@ static void modify_u32(char *com, const char *prompt,
 		       const char *format, __u32 *val)
 {
 	char buf[200];
-	u_long v;
+	unsigned long v;
 	char *tmp;
 
 	sprintf(buf, format, *val);
@@ -617,7 +625,7 @@ static void modify_u32(char *com, const char *prompt,
 		buf[strlen (buf) - 1] = '\0';
 	if (!buf[0])
 		return;
-	v = strtol(buf, &tmp, 0);
+	v = strtoul(buf, &tmp, 0);
 	if (*tmp)
 		com_err(com, 0, "Bad value - %s", buf);
 	else
@@ -719,78 +727,6 @@ void do_modify_inode(int argc, char *argv[])
 	}
 }
 
-/*
- * list directory
- */
-
-struct list_dir_struct {
-	FILE	*f;
-	int	col;
-};
-
-static int list_dir_proc(struct ext2_dir_entry *dirent,
-			 int	offset,
-			 int	blocksize,
-			 char	*buf,
-			 void	*private)
-{
-	char	name[EXT2_NAME_LEN];
-	char	tmp[EXT2_NAME_LEN + 16];
-
-	struct list_dir_struct *ls = (struct list_dir_struct *) private;
-	int	thislen;
-
-	thislen = (dirent->name_len < EXT2_NAME_LEN) ? dirent->name_len :
-		EXT2_NAME_LEN;
-	strncpy(name, dirent->name, thislen);
-	name[thislen] = '\0';
-
-	sprintf(tmp, "%d (%d) %s   ", dirent->inode, dirent->rec_len, name);
-	thislen = strlen(tmp);
-
-	if (ls->col + thislen > 80) {
-		fprintf(ls->f, "\n");
-		ls->col = 0;
-	}
-	fprintf(ls->f, "%s", tmp);
-	ls->col += thislen;
-		
-	return 0;
-}
-
-void do_list_dir(int argc, char *argv[])
-{
-	ino_t	inode;
-	int	retval;
-	struct list_dir_struct ls;
-	
-	if (argc > 2) {
-		com_err(argv[0], 0, "Usage: list_dir [pathname]");
-		return;
-	}
-	if (check_fs_open(argv[0]))
-		return;
-
-	if (argc == 2)
-		inode = string_to_inode(argv[1]);
-	else
-		inode = cwd;
-	if (!inode)
-		return;
-
-	ls.f = open_pager();
-	ls.col = 0;
-	retval = ext2fs_dir_iterate(current_fs, inode,
-				    DIRENT_FLAG_INCLUDE_EMPTY,
-				    0, list_dir_proc, &ls);
-	fprintf(ls.f, "\n");
-	close_pager(ls.f);
-	if (retval)
-		com_err(argv[1], retval, "");
-
-	return;
-}
-
 void do_change_working_dir(int argc, char *argv[])
 {
 	ino_t	inode;
@@ -1329,6 +1265,7 @@ static void kill_file_by_inode(ino_t inode)
 	printf("Kill file by inode %ld\n", inode);
 	ext2fs_block_iterate(current_fs, inode, 0, NULL,
 			     release_blocks_proc, NULL);
+	printf("\n");
 	ext2fs_unmark_inode_bitmap(current_fs->inode_map, inode);
 
 	ext2fs_mark_bb_dirty(current_fs);
@@ -1471,7 +1408,7 @@ void main(int argc, char **argv)
 {
 	int		retval;
 	int		sci_idx;
-	const char	*usage = "Usage: debugfs [-w] [device]";
+	const char	*usage = "Usage: debugfs [[-w] device]";
 	char		c;
 	int		open_flags = 0;
 	char		*request = 0;
