@@ -1,8 +1,12 @@
 /*
  * pass1.c -- pass #1 of e2fsck: sequential scan of the inode table
  * 
- * Copyright (C) 1993, 1994 Theodore Ts'o.  This file may be
- * redistributed under the terms of the GNU Public License.
+ * Copyright (C) 1993, 1994, 1995, 1996, 1997 Theodore Ts'o.
+ *
+ * %Begin-Header%
+ * This file may be redistributed under the terms of the GNU Public
+ * License.
+ * %End-Header%
  * 
  * Pass 1 of e2fsck iterates over all the inodes in the filesystems,
  * and applies the following tests to each inode:
@@ -16,6 +20,7 @@
  * 	- A bitmap of which inodes are in use.		(inode_used_map)
  * 	- A bitmap of which inodes are directories.	(inode_dir_map)
  * 	- A bitmap of which inodes have bad fields.	(inode_bad_map)
+ * 	- A bitmap of which inodes are in bad blocks.	(inode_bb_map)
  * 	- A bitmap of which blocks are in use.		(block_found_map)
  * 	- A bitmap of which blocks are in use by two inodes	(block_dup_map)
  * 	- The data blocks of the directory inodes.	(dir_map)
@@ -37,6 +42,7 @@
 
 #include <et/com_err.h>
 #include "e2fsck.h"
+#include "problem.h"
 
 #ifdef NO_INLINE_FUNCS
 #define _INLINE_
@@ -64,28 +70,31 @@ int fs_fragmented = 0;
 ext2fs_inode_bitmap inode_used_map = 0;	/* Inodes which are in use */
 ext2fs_inode_bitmap inode_bad_map = 0;	/* Inodes which are bad in some way */
 ext2fs_inode_bitmap inode_dir_map = 0;	/* Inodes which are directories */
+ext2fs_inode_bitmap inode_bb_map = 0;	/* Inodes which are in bad blocks */
 
 ext2fs_block_bitmap block_found_map = 0;
 ext2fs_block_bitmap block_dup_map = 0;
 ext2fs_block_bitmap block_illegal_map = 0;
 
-unsigned short * inode_link_info = NULL;
+ext2_icount_t inode_link_info = 0;	
 
 static int process_block(ext2_filsys fs, blk_t	*blocknr,
-			 int	blockcnt, void	*private);
+			 int	blockcnt, blk_t ref_blk, 
+			 int ref_offset, void *private);
 static int process_bad_block(ext2_filsys fs, blk_t *block_nr,
-			     int blockcnt, void *private);
-static void check_blocks(ext2_filsys fs, ino_t ino, struct ext2_inode *inode,
+			     int blockcnt, blk_t ref_blk,
+			     int ref_offset, void *private);
+static void check_blocks(ext2_filsys fs, struct problem_context *pctx,
 			 char *block_buf);
 static void mark_table_blocks(ext2_filsys fs);
 static void alloc_bad_map(ext2_filsys fs);
+static void alloc_bb_map(ext2_filsys fs);
 static void handle_fs_bad_blocks(ext2_filsys fs);
 static void process_inodes(ext2_filsys fs, char *block_buf);
 static int process_inode_cmp(const void *a, const void *b);
-static int dir_block_cmp(const void *a, const void *b);
 static errcode_t scan_callback(ext2_filsys fs, ext2_inode_scan scan,
 				  dgrp_t group, void * private);
-static char *describe_illegal_block(ext2_filsys fs, blk_t block);
+/* static char *describe_illegal_block(ext2_filsys fs, blk_t block); */
 
 struct process_block_struct {
 	ino_t	ino;
@@ -93,9 +102,9 @@ struct process_block_struct {
 	int	num_blocks;
 	int	last_block;
 	int	num_illegal_blocks;
-	int	fix;
 	blk_t	previous_block;
 	struct ext2_inode *inode;
+	struct problem_context *pctx;
 };
 
 struct process_inode_block {
@@ -116,13 +125,6 @@ static struct process_inode_block *inodes_to_process;
 static int process_inode_count;
 int process_inode_size = 256;
 
-/*
- * For the directory blocks list.
- */
-struct dir_block_struct *dir_blocks = 0;
-int	dir_block_count = 0;
-int	dir_block_size = 0;
-
 /*
  * Free all memory allocated by pass1 in preparation for restarting
  * things.
@@ -132,13 +134,19 @@ static void unwind_pass1(ext2_filsys fs)
 	ext2fs_free_inode_bitmap(inode_used_map);	inode_used_map = 0;
 	ext2fs_free_inode_bitmap(inode_dir_map);	inode_dir_map = 0;
 	ext2fs_free_block_bitmap(block_found_map);	block_found_map = 0;
-	free(inode_link_info);	inode_link_info = 0;
+	ext2fs_free_icount(inode_link_info); inode_link_info = 0;
 	free(inodes_to_process);inodes_to_process = 0;
-	free(dir_blocks);	dir_blocks = 0;
-	dir_block_size = 0;
+	ext2fs_free_dblist(fs->dblist);	fs->dblist = 0;
+	free_dir_info(fs);
 	if (block_dup_map) {
 		ext2fs_free_block_bitmap(block_dup_map); block_dup_map = 0;
 	}
+	if (inode_bb_map) {
+		ext2fs_free_inode_bitmap(inode_bb_map); inode_bb_map = 0;
+	}
+	if (inode_bad_map) {
+		ext2fs_free_inode_bitmap(inode_bad_map); inode_bad_map = 0;
+	}
 
 	/* Clear statistic counters */
 	fs_directory_count = 0;
@@ -167,6 +175,7 @@ void pass1(ext2_filsys fs)
 	errcode_t	retval;
 	struct resource_track	rtrack;
 	unsigned char	frag, fsize;
+	struct		problem_context pctx;
 	
 	init_resource_track(&rtrack);
 	
@@ -208,19 +217,23 @@ void pass1(ext2_filsys fs)
 			"while allocating block_illegal_map");
 		fatal_error(0);
 	}
-	inode_link_info = allocate_memory((fs->super->s_inodes_count + 1) *
-					  sizeof(unsigned short),
-					  "inode link count array");
+	retval = ext2fs_create_icount2(fs, 0, 0, 0, &inode_link_info);
+	if (retval) {
+		com_err("ext2fs_create_icount", retval,
+			"while creating inode_link_info");
+		fatal_error(0);
+	}
 	inodes_to_process = allocate_memory(process_inode_size *
 					    sizeof(struct process_inode_block),
 					    "array of inodes to process");
 	process_inode_count = 0;
 
-	dir_block_size = get_num_dirs(fs) * 4;
-	dir_block_count = 0;
-	dir_blocks = allocate_memory(sizeof(struct dir_block_struct) *
-				     dir_block_size,
-				     "directory block information");
+	retval = ext2fs_init_dblist(fs, 0);
+	if (retval) {
+		com_err(program_name, retval,
+			"while allocating directory block information");
+		fatal_error(0);
+	}
 
 	mark_table_blocks(fs);
 	block_buf = allocate_memory(fs->blocksize * 3, "block interate buffer");
@@ -234,6 +247,7 @@ void pass1(ext2_filsys fs)
 		com_err(program_name, retval, "while opening inode scan");
 		fatal_error(0);
 	}
+	ext2fs_inode_scan_flags(scan, EXT2_SF_SKIP_MISSING_ITABLE, 0);
 	retval = ext2fs_get_next_inode(scan, &ino, &inode);
 	if (retval) {
 		com_err(program_name, retval, "while starting inode scan");
@@ -241,25 +255,36 @@ void pass1(ext2_filsys fs)
 	}
 	stashed_inode = &inode;
 	ext2fs_set_inode_callback(scan, scan_callback, block_buf);
+	clear_problem_context(&pctx);
 	while (ino) {
 		stashed_ino = ino;
-		inode_link_info[ino] = inode.i_links_count;
+		if (inode.i_links_count)
+			retval = ext2fs_icount_store(inode_link_info, ino, 
+						     inode.i_links_count);
+		if (retval) {
+			com_err("ext2fs_icount_fetch", retval,
+				"while adding inode %u", ino);
+			fatal_error(0);
+		}
+		pctx.ino = ino;
+		pctx.inode = &inode;
 		if (ino == EXT2_BAD_INO) {
 			struct process_block_struct pb;
 			
 			pb.ino = EXT2_BAD_INO;
 			pb.num_blocks = pb.last_block = 0;
 			pb.num_illegal_blocks = 0;
-			pb.suppress = pb.clear = pb.is_dir = 0;
+			pb.suppress = 0; pb.clear = 0; pb.is_dir = 0;
 			pb.fragmented = 0;
-			pb.fix = -1;
 			pb.inode = &inode;
-			retval = ext2fs_block_iterate(fs, ino, 0, block_buf,
-						      process_bad_block, &pb);
+			pb.pctx = &pctx;
+			retval = ext2fs_block_iterate2(fs, ino, 0, block_buf,
+						       process_bad_block, &pb);
 			if (retval)
 				com_err(program_name, retval, "while calling e2fsc_block_interate in pass 1");
 
 			ext2fs_mark_inode_bitmap(inode_used_map, ino);
+			clear_problem_context(&pctx);
 			goto next;
 		}
 		if (ino == EXT2_ROOT_INO) {
@@ -269,16 +294,14 @@ void pass1(ext2_filsys fs)
 			 * regnerated in pass #3.
 			 */
 			if (!LINUX_S_ISDIR(inode.i_mode)) {
-				printf("Root inode is not a directory.  ");
-				preenhalt(fs);
-				if (ask("Clear", 1)) {
+				if (fix_problem(fs, PR_1_ROOT_NO_DIR, &pctx)) {
 					inode.i_dtime = time(0);
 					inode.i_links_count = 0;
-					inode_link_info[ino] = 0;
+					ext2fs_icount_store(inode_link_info,
+							    ino, 0);
 					e2fsck_write_inode(fs, ino, &inode,
 							   "pass1");
-				} else 
-					ext2fs_unmark_valid(fs);
+				}
 			}
 			/*
 			 * If dtime is set, offer to clear it.  mke2fs
@@ -290,34 +313,30 @@ void pass1(ext2_filsys fs)
 			 * as a special case.
 			 */
 			if (inode.i_dtime && inode.i_links_count) {
-				if (ask("Root inode has dtime set "
-					"(probably due to old mke2fs).  Fix",
-					1)) {
+				if (fix_problem(fs, PR_1_ROOT_DTIME, &pctx)) {
 					inode.i_dtime = 0;
 					e2fsck_write_inode(fs, ino, &inode,
 							   "pass1");
-				} else
-					ext2fs_unmark_valid(fs);
+				}
 			}
 		}
 		if (ino == EXT2_BOOT_LOADER_INO) {
 			ext2fs_mark_inode_bitmap(inode_used_map, ino);
-			check_blocks(fs, ino, &inode, block_buf);
+			check_blocks(fs, &pctx, block_buf);
 			goto next;
 		}
 		if ((ino != EXT2_ROOT_INO) &&
 		    (ino < EXT2_FIRST_INODE(fs->super))) {
 			ext2fs_mark_inode_bitmap(inode_used_map, ino);
 			if (inode.i_mode != 0) {
-				printf("Reserved inode %lu has bad mode.  ", ino);
-				if (ask("Clear", 1)) {
+				if (fix_problem(fs,
+					    PR_1_RESERVED_BAD_MODE, &pctx)) {
 					inode.i_mode = 0;
 					e2fsck_write_inode(fs, ino, &inode,
 							   "pass1");
-				} else 
-					ext2fs_unmark_valid(fs);
+				}
 			}
-			check_blocks(fs, ino, &inode, block_buf);
+			check_blocks(fs, &pctx, block_buf);
 			goto next;
 		}
 		/*
@@ -326,14 +345,12 @@ void pass1(ext2_filsys fs)
 		 */
 		if (!inode.i_links_count) {
 			if (!inode.i_dtime && inode.i_mode) {
-				printf("Deleted inode %lu has zero dtime.\n",
-				       ino);
-				if (ask("Set dtime", 1)) {
+				if (fix_problem(fs,
+					    PR_1_ZERO_DTIME, &pctx)) {
 					inode.i_dtime = time(0);
 					e2fsck_write_inode(fs, ino, &inode,
 							   "pass1");
-				} else
-					ext2fs_unmark_valid(fs);
+				}
 			}
 			goto next;
 		}
@@ -348,13 +365,10 @@ void pass1(ext2_filsys fs)
 		 * 
 		 */
 		if (inode.i_dtime) {
-			printf("Inode %lu is in use, but has dtime set\n",
-			       ino);
-			if (ask("Clear dtime", 1)) {
+			if (fix_problem(fs, PR_1_SET_DTIME, &pctx)) {
 				inode.i_dtime = 0;
 				e2fsck_write_inode(fs, ino, &inode, "pass1");
-			} else
-				ext2fs_unmark_valid(fs);
+			}
 		}
 		
 		ext2fs_mark_inode_bitmap(inode_used_map, ino);
@@ -384,7 +398,7 @@ void pass1(ext2_filsys fs)
 		
 		if (LINUX_S_ISDIR(inode.i_mode)) {
 			ext2fs_mark_inode_bitmap(inode_dir_map, ino);
-			add_dir_info(fs, ino, 0, &inode);
+			add_dir_info(fs, ino, 0);
 			fs_directory_count++;
 		} else if (LINUX_S_ISREG (inode.i_mode))
 			fs_regular_count++;
@@ -394,8 +408,10 @@ void pass1(ext2_filsys fs)
 			fs_blockdev_count++;
 		else if (LINUX_S_ISLNK (inode.i_mode)) {
 			fs_symlinks_count++;
-			if (!inode.i_blocks)
+			if (!inode.i_blocks) {
 				fs_fast_symlinks_count++;
+				goto next;
+			}
 		}
 		else if (LINUX_S_ISFIFO (inode.i_mode))
 			fs_fifo_count++;
@@ -419,12 +435,19 @@ void pass1(ext2_filsys fs)
 			inodes_to_process[process_inode_count].inode = inode;
 			process_inode_count++;
 		} else
-			check_blocks(fs, ino, &inode, block_buf);
+			check_blocks(fs, &pctx, block_buf);
 
 		if (process_inode_count >= process_inode_size)
 			process_inodes(fs, block_buf);
 	next:
 		retval = ext2fs_get_next_inode(scan, &ino, &inode);
+		if (retval == EXT2_ET_BAD_BLOCK_IN_INODE_TABLE) {
+			if (!inode_bb_map)
+				alloc_bb_map(fs);
+			ext2fs_mark_inode_bitmap(inode_bb_map, ino);
+			ext2fs_mark_inode_bitmap(inode_used_map, ino);
+			goto next;
+		}
 		if (retval) {
 			com_err(program_name, retval,
 				"while doing inode scan");
@@ -435,9 +458,6 @@ void pass1(ext2_filsys fs)
 	ext2fs_close_inode_scan(scan);
 	ehandler_operation(0);
 
-	qsort(dir_blocks, dir_block_count, sizeof(struct dir_block_struct),
-	      dir_block_cmp);
-
 	if (invalid_bitmaps)
 		handle_fs_bad_blocks(fs);
 
@@ -463,7 +483,7 @@ endit:
 	free(block_buf);
 	ext2fs_free_block_bitmap(block_illegal_map);
 	block_illegal_map = 0;
-	
+
 	if (tflag > 1) {
 		printf("Pass 1: ");
 		print_resource_track(&rtrack);
@@ -488,30 +508,33 @@ static void process_inodes(ext2_filsys fs, char *block_buf)
 {
 	int			i;
 	struct ext2_inode	*old_stashed_inode;
-	ino_t			ino;
+	ino_t			old_stashed_ino;
 	const char		*old_operation;
 	char			buf[80];
-
+	struct problem_context	pctx;
+	
 #if 0
 	printf("begin process_inodes: ");
 #endif
 	old_operation = ehandler_operation(0);
 	old_stashed_inode = stashed_inode;
+	old_stashed_ino = stashed_ino;
 	qsort(inodes_to_process, process_inode_count,
 		      sizeof(struct process_inode_block), process_inode_cmp);
+	clear_problem_context(&pctx);
 	for (i=0; i < process_inode_count; i++) {
-		stashed_inode = &inodes_to_process[i].inode;
-		ino = inodes_to_process[i].ino;
-		stashed_ino = ino;
-#if 0
-		printf("%u ", ino);
-#endif
-		sprintf(buf, "reading indirect blocks of inode %lu", ino);
-		ehandler_operation(buf);
-		check_blocks(fs, ino, stashed_inode, block_buf);
+		pctx.inode = stashed_inode = &inodes_to_process[i].inode;
+		pctx.ino = stashed_ino = inodes_to_process[i].ino;
 		
+#if 0
+		printf("%u ", pctx.ino);
+#endif
+		sprintf(buf, "reading indirect blocks of inode %lu", pctx.ino);
+		ehandler_operation(buf);
+		check_blocks(fs, &pctx, block_buf);
 	}
 	stashed_inode = old_stashed_inode;
+	stashed_ino = old_stashed_ino;
 	process_inode_count = 0;
 #if 0
 	printf("end process inodes\n");
@@ -530,22 +553,6 @@ static int process_inode_cmp(const void *a, const void *b)
 		ib_b->inode.i_block[EXT2_IND_BLOCK]);
 }
 
-static int dir_block_cmp(const void *a, const void *b)
-{
-	const struct dir_block_struct *db_a =
-		(const struct dir_block_struct *) a;
-	const struct dir_block_struct *db_b =
-		(const struct dir_block_struct *) b;
-
-	if (db_a->blk != db_b->blk)
-		return (db_a->blk - db_b->blk);
-	
-	if (db_a->ino != db_b->ino)
-		return (db_a->ino - db_b->ino);
-
-	return (db_a->blockcnt - db_b->blockcnt);
-}
-
 /*
  * This procedure will allocate the inode bad map table
  */
@@ -562,6 +569,22 @@ static void alloc_bad_map(ext2_filsys fs)
 	}
 }
 
+/*
+ * This procedure will allocate the inode "bb" (badblock) map table
+ */
+static void alloc_bb_map(ext2_filsys fs)
+{
+	errcode_t	retval;
+	
+	retval = ext2fs_allocate_inode_bitmap(fs, "inode in bad block map",
+					      &inode_bb_map);
+	if (retval) {
+		com_err("ext2fs_allocate_inode_bitmap", retval,
+			"while allocating inode in bad block map");
+		fatal_error(0);
+	}
+}
+
 /*
  * Marks a block as in use, setting the dup_map if it's been set
  * already.  Called by process_block and process_bad_block.
@@ -593,27 +616,30 @@ static _INLINE_ void mark_block_used(ext2_filsys fs, blk_t block)
  * This subroutine is called on each inode to account for all of the
  * blocks used by that inode.
  */
-static void check_blocks(ext2_filsys fs, ino_t ino, struct ext2_inode *inode,
+static void check_blocks(ext2_filsys fs, struct problem_context *pctx,
 			 char *block_buf)
 {
 	struct process_block_struct pb;
 	errcode_t	retval;
+	ino_t		ino = pctx->ino;
+	struct ext2_inode *inode = pctx->inode;
 	
-	if (!inode_has_valid_blocks(inode))
+	if (!ext2fs_inode_has_valid_blocks(pctx->inode))
 		return;
 	
 	pb.ino = ino;
 	pb.num_blocks = pb.last_block = 0;
 	pb.num_illegal_blocks = 0;
-	pb.suppress = pb.clear = 0;
+	pb.suppress = 0; pb.clear = 0;
 	pb.fragmented = 0;
 	pb.previous_block = 0;
-	pb.is_dir = LINUX_S_ISDIR(inode->i_mode);
-	pb.fix = -1;
+	pb.is_dir = LINUX_S_ISDIR(pctx->inode->i_mode);
 	pb.inode = inode;
-	retval = ext2fs_block_iterate(fs, ino,
-				      pb.is_dir ? BLOCK_FLAG_HOLE : 0,
-				      block_buf, process_block, &pb);
+	pb.pctx = pctx;
+	retval = ext2fs_block_iterate2(fs, ino,
+				       pb.is_dir ? BLOCK_FLAG_HOLE : 0,
+				       block_buf, process_block, &pb);
+	reset_problem_latch(PR_LATCH_BLOCK);
 	if (retval)
 		com_err(program_name, retval,
 			"while calling ext2fs_block_iterate in check_blocks");
@@ -629,7 +655,7 @@ static void check_blocks(ext2_filsys fs, ino_t ino, struct ext2_inode *inode,
 			fatal_error(0);
 		}
 		inode->i_links_count = 0;
-		inode_link_info[ino] = 0;
+		ext2fs_icount_store(inode_link_info, ino, 0);
 		inode->i_dtime = time(0);
 		e2fsck_write_inode(fs, ino, inode, "check_blocks");
 		ext2fs_unmark_inode_bitmap(inode_dir_map, ino);
@@ -643,9 +669,6 @@ static void check_blocks(ext2_filsys fs, ino_t ino, struct ext2_inode *inode,
 		return;
 	}
 
-	if (pb.fix > 0)
-		e2fsck_read_inode(fs, ino, inode, "check_blocks");
-
 	pb.num_blocks *= (fs->blocksize / 512);
 #if 0
 	printf("inode %u, i_size = %lu, last_block = %lu, i_blocks=%lu, num_blocks = %lu\n",
@@ -653,41 +676,38 @@ static void check_blocks(ext2_filsys fs, ino_t ino, struct ext2_inode *inode,
 	       pb.num_blocks);
 #endif
 	if (!pb.num_blocks && pb.is_dir) {
-		printf("Inode %lu is a zero length directory.  ", ino);
-		if (ask("Clear", 1)) {
+		if (fix_problem(fs, PR_1_ZERO_LENGTH_DIR, pctx)) {
 			inode->i_links_count = 0;
-			inode_link_info[ino] = 0;
+			ext2fs_icount_store(inode_link_info, ino, 0);
 			inode->i_dtime = time(0);
 			e2fsck_write_inode(fs, ino, inode, "check_blocks");
 			ext2fs_unmark_inode_bitmap(inode_dir_map, ino);
 			ext2fs_unmark_inode_bitmap(inode_used_map, ino);
 			fs_directory_count--;
 			pb.is_dir = 0;
-		} else
-			ext2fs_unmark_valid(fs);
+		}
 	}
-	if ((pb.is_dir && (inode->i_size != (pb.last_block + 1) * fs->blocksize)) ||
+	if ((pb.is_dir && (inode->i_size !=
+			   (pb.last_block + 1) * fs->blocksize)) ||
 	    (inode->i_size < pb.last_block * fs->blocksize)) {
-		printf ("%s %lu, incorrect size, %u (counted = %u). ",
-			pb.is_dir ? "Directory" : "Inode", ino, inode->i_size,
-			(pb.last_block+1) * fs->blocksize);
-		if (ask ("Set size to counted", 1)) {
-			inode->i_size = (pb.last_block+1) * fs->blocksize;
+		pctx->num = (pb.last_block+1) * fs->blocksize;
+		if (fix_problem(fs, PR_1_BAD_I_SIZE, pctx)) {
+			inode->i_size = pctx->num;
 			e2fsck_write_inode(fs, ino, inode, "check_blocks");
-		} else
-			ext2fs_unmark_valid(fs);
+		}
+		pctx->num = 0;
 	}
 	if (pb.num_blocks != inode->i_blocks) {
-		printf ("Inode %lu, i_blocks wrong %u (counted=%u).  ",
-			ino, inode->i_blocks, pb.num_blocks);
-		if (ask ("Set i_blocks to counted", 1)) {
+		pctx->num = pb.num_blocks;
+		if (fix_problem(fs, PR_1_BAD_I_BLOCKS, pctx)) {
 			inode->i_blocks = pb.num_blocks;
 			e2fsck_write_inode(fs, ino, inode, "check_blocks");
-		} else
-				ext2fs_unmark_valid(fs);
+		}
+		pctx->num = 0;
 	}
 }
 
+#if 0
 /*
  * Helper function called by process block when an illegal block is
  * found.  It returns a description about why the block is illegal
@@ -737,6 +757,7 @@ static char *describe_illegal_block(ext2_filsys fs, blk_t block)
 	}
 	return(problem);
 }
+#endif
 
 /*
  * This is a helper function for check_blocks().
@@ -744,14 +765,19 @@ static char *describe_illegal_block(ext2_filsys fs, blk_t block)
 int process_block(ext2_filsys fs,
 		  blk_t	*block_nr,
 		  int blockcnt,
+		  blk_t ref_block,
+		  int ref_offset, 
 		  void *private)
 {
 	struct process_block_struct *p;
-	char	*problem;
+	struct problem_context *pctx;
 	blk_t	blk = *block_nr;
 	int	ret_code = 0;
+	int	problem = 0;
+	errcode_t	retval;
 
 	p = (struct process_block_struct *) private;
+	pctx = p->pctx;
 
 	if (blk == 0) {
 		if (p->is_dir == 0) {
@@ -767,8 +793,10 @@ int process_block(ext2_filsys fs,
 		if (blockcnt < 0)
 			return 0;
 		if (blockcnt * fs->blocksize < p->inode->i_size) {
-			printf("Hole found in directory inode %lu!  "
-			       "(blkcnt=%d)\n", p->ino, blockcnt);
+#if 0
+			printf("Missing block (#%d) in directory inode %lu!\n",
+			       blockcnt, p->ino);
+#endif
 			goto mark_dir;
 		}
 		return 0;
@@ -790,43 +818,37 @@ int process_block(ext2_filsys fs,
 	}
 	p->previous_block = blk;
 	
-	
 	if (blk < fs->super->s_first_data_block ||
-	    blk >= fs->super->s_blocks_count ||
-	    ext2fs_test_block_bitmap(block_illegal_map, blk)) {
-		problem = describe_illegal_block(fs, blk);
-		if (preen) {
-			printf("Block %u of inode %lu %s\n", blk, p->ino,
-			       problem);
-			preenhalt(fs);
-		}
-		if (p->fix == -1) {
-			printf("Remove illegal block(s) in inode %lu", p->ino);
-			p->fix = ask("", 1);
-		}
+	    blk >= fs->super->s_blocks_count)
+		problem = PR_1_ILLEGAL_BLOCK_NUM;
+#if 0
+	else
+		if (ext2fs_test_block_bitmap(block_illegal_map, blk))
+			problem = PR_1_BLOCK_OVERLAPS_METADATA;
+#endif
+
+	if (problem) {
 		p->num_illegal_blocks++;
-		if (!p->suppress && (p->num_illegal_blocks % 20) == 0) {
-			printf("Too many illegal blocks in inode %lu.\n",
-			       p->ino);
-			if (ask("Clear inode", 1)) {
+		if (!p->suppress && (p->num_illegal_blocks % 12) == 0) {
+			if (fix_problem(fs, PR_1_TOO_MANY_BAD_BLOCKS, pctx)) {
 				p->clear = 1;
 				return BLOCK_ABORT;
 			}
-			if (ask("Supress messages", 0)) {
+			if (ask("Suppress messages", 0)) {
 				p->suppress = 1;
+				suppress_latch_group(PR_LATCH_BLOCK, 1);
 			}
 		}
-		if (!p->suppress)
-			printf("Block #%d (%u) %s.  %s\n", blockcnt, blk,
-			       problem, clear_msg[p->fix]);
-		if (p->fix) {
+		pctx->blk = blk;
+		pctx->blkcount = blockcnt;
+		if (fix_problem(fs, problem, pctx)) {
 			blk = *block_nr = 0;
 			ret_code = BLOCK_CHANGED;
 			goto mark_dir;
-		} else {
-			ext2fs_unmark_valid(fs);
+		} else
 			return 0;
-		}
+		pctx->blk = 0;
+		pctx->blkcount = -1;
 	}
 
 	mark_block_used(fs, blk);
@@ -835,20 +857,13 @@ int process_block(ext2_filsys fs,
 		p->last_block = blockcnt;
 mark_dir:
 	if (p->is_dir && (blockcnt >= 0)) {
-		if (dir_block_count >= dir_block_size) {
-			dir_block_size += 100;
-			dir_blocks = realloc(dir_blocks,
-					     dir_block_size *
-					     sizeof(struct dir_block_struct));
-			if (dir_blocks == 0)
-				fatal_error("Not enough memory to "
-					    "realloc dir_blocks");
+		retval = ext2fs_add_dir_block(fs->dblist, p->ino,
+					      blk, blockcnt);
+		if (retval) {
+			com_err(program_name, retval,
+				"while adding to directory block list");
+			fatal_error(0);
 		}
-
-		dir_blocks[dir_block_count].blk = blk;
-		dir_blocks[dir_block_count].ino = p->ino;
-		dir_blocks[dir_block_count].blockcnt = blockcnt;
-		dir_block_count++;
 	}
 	return ret_code;
 }
@@ -886,34 +901,33 @@ static int bad_primary_block(ext2_filsys fs, blk_t *block_nr)
 int process_bad_block(ext2_filsys fs,
 		      blk_t *block_nr,
 		      int blockcnt,
+		      blk_t ref_block,
+		      int ref_offset,
 		      void *private)
 {
 	struct process_block_struct *p;
 	blk_t		blk = *block_nr;
 	int		first_block;
 	int		i;
-	
+	struct problem_context *pctx;
+
 	if (!blk)
 		return 0;
+	
 	p = (struct process_block_struct *) private;
+	pctx = p->pctx;
+	
+	pctx->blk = blk;
+	pctx->blkcount = blockcnt;
+	
 
 	if ((blk < fs->super->s_first_data_block) ||
 	    (blk >= fs->super->s_blocks_count)) {
-		if (preen) {
-			printf("Illegal block %u in bad block inode\n", blk);
-			preenhalt(fs);
-		}
-		if (p->fix == -1)
-			p->fix = ask("Remove illegal block(s) in bad block inode", 1);
-		printf("Illegal block %u in bad block inode.  %s\n", blk,
-		       clear_msg[p->fix]);
-		if (p->fix) {
+		if (fix_problem(fs, PR_1_BB_ILLEGAL_BLOCK_NUM, pctx)) {
 			*block_nr = 0;
 			return BLOCK_CHANGED;
-		} else {
-			ext2fs_unmark_valid(fs);
+		} else
 			return 0;
-		}
 	}
 
 	if (blockcnt < 0) {
@@ -943,6 +957,7 @@ int process_bad_block(ext2_filsys fs,
 	first_block = fs->super->s_first_data_block;
 	
 	for (i = 0; i < fs->group_desc_count; i++ ) {
+		pctx->group = i;
 		if (blk == first_block) {
 			if (i == 0) {
 				printf("The primary superblock (%u) is "
@@ -973,39 +988,29 @@ int process_bad_block(ext2_filsys fs,
 			return 0;
 		}
 		if (blk == fs->group_desc[i].bg_block_bitmap) {
-			printf("Group %d's block bitmap (%u) is bad.  ",
-			       i, blk);
-			if (ask("Relocate", 1)) {
+			if (fix_problem(fs, PR_1_BB_BAD_BLOCK, pctx)) {
 				invalid_block_bitmap[i]++;
 				invalid_bitmaps++;
-			} else
-				ext2fs_unmark_valid(fs);
+			}
 			return 0;
 		}
 		if (blk == fs->group_desc[i].bg_inode_bitmap) {
-			printf("Group %d's inode bitmap (%u) is bad.  ",
-			       i, blk);
-			if (ask("Relocate", 1)) {
+			if (fix_problem(fs, PR_1_IB_BAD_BLOCK, pctx)) {
 				invalid_inode_bitmap[i]++;
 				invalid_bitmaps++;
-			} else
-				ext2fs_unmark_valid(fs);
+			}
 			return 0;
 		}
 		if ((blk >= fs->group_desc[i].bg_inode_table) &&
 		    (blk < (fs->group_desc[i].bg_inode_table +
 			    fs->inode_blocks_per_group))) {
-			printf("WARNING: Severe data loss possible!!!!\n");
-			printf("Bad block %u in group %d's inode table.  ",
-			       blk, i);
-			if (ask("Relocate", 1)) {
-				invalid_inode_table[i]++;
-				invalid_bitmaps++;
-			} else
-				ext2fs_unmark_valid(fs);
+			/*
+			 * If there are bad blocks in the inode table,
+			 * the inode scan code will try to do
+			 * something reasonable automatically.
+			 */
 			return 0;
 		}
-		first_block += fs->super->s_blocks_per_group;
 	}
 	/*
 	 * If we've gotten to this point, then the only
@@ -1114,24 +1119,47 @@ static void mark_table_blocks(ext2_filsys fs)
 {
 	blk_t	block, b;
 	int	i,j;
+	struct problem_context pctx;
+	
+	clear_problem_context(&pctx);
 	
 	block = fs->super->s_first_data_block;
 	for (i = 0; i < fs->group_desc_count; i++) {
+		pctx.group = i;
+		/*
+		 * Mark the blocks used for the inode table
+		 */
+		if (fs->group_desc[i].bg_inode_table) {
+			for (j = 0, b = fs->group_desc[i].bg_inode_table;
+			     j < fs->inode_blocks_per_group;
+			     j++, b++) {
+				if (ext2fs_test_block_bitmap(block_found_map,
+							     b)) {
+					pctx.blk = b;
+					if (fix_problem(fs,
+						PR_1_ITABLE_CONFLICT, &pctx)) {
+						invalid_inode_table[i]++;
+						invalid_bitmaps++;
+					}
+				} else {
+				    ext2fs_mark_block_bitmap(block_found_map,
+							     b);
+				    ext2fs_mark_block_bitmap(block_illegal_map,
+							     b);
+			    	}
+			}
+		}
+			    
 		/*
 		 * Mark block used for the block bitmap 
 		 */
 		if (fs->group_desc[i].bg_block_bitmap) {
 			if (ext2fs_test_block_bitmap(block_found_map,
 				     fs->group_desc[i].bg_block_bitmap)) {
-				printf("Group %i's block bitmap at %u "
-				       "conflicts with some other fs block.\n",
-				       i, fs->group_desc[i].bg_block_bitmap);
-				preenhalt(fs);
-				if (ask("Relocate", 1)) {
+				pctx.blk = fs->group_desc[i].bg_block_bitmap;
+				if (fix_problem(fs, PR_1_BB_CONFLICT, &pctx)) {
 					invalid_block_bitmap[i]++;
 					invalid_bitmaps++;
-				} else {
-					ext2fs_unmark_valid(fs);
 				}
 			} else {
 			    ext2fs_mark_block_bitmap(block_found_map,
@@ -1147,16 +1175,11 @@ static void mark_table_blocks(ext2_filsys fs)
 		if (fs->group_desc[i].bg_inode_bitmap) {
 			if (ext2fs_test_block_bitmap(block_found_map,
 				     fs->group_desc[i].bg_inode_bitmap)) {
-				printf("Group %i's inode bitmap at %u "
-				       "conflicts with some other fs block.\n",
-				       i, fs->group_desc[i].bg_inode_bitmap);
-				preenhalt(fs);
-				if (ask("Relocate", 1)) {
+				pctx.blk = fs->group_desc[i].bg_inode_bitmap;
+				if (fix_problem(fs, PR_1_IB_CONFLICT, &pctx)) {
 					invalid_inode_bitmap[i]++;
 					invalid_bitmaps++;
-				} else {
-					ext2fs_unmark_valid(fs);
-				}
+				} 
 			} else {
 			    ext2fs_mark_block_bitmap(block_found_map,
 				     fs->group_desc[i].bg_inode_bitmap);
@@ -1164,50 +1187,23 @@ static void mark_table_blocks(ext2_filsys fs)
 				     fs->group_desc[i].bg_inode_bitmap);
 			}
 		}
-		    
-		/*
-		 * Mark the blocks used for the inode table
-		 */
-		if (fs->group_desc[i].bg_inode_table) {
-			for (j = 0, b = fs->group_desc[i].bg_inode_table;
-			     j < fs->inode_blocks_per_group;
-			     j++, b++) {
-				if (ext2fs_test_block_bitmap(block_found_map,
-							     b)) {
-					printf("Group %i's inode table at %u "
-					       "conflicts with some other "
-					       "fs block.\n",
-					       i, b);
-					preenhalt(fs);
-					if (ask("Relocate", 1)) {
-						invalid_inode_table[i]++;
-						invalid_bitmaps++;
-					} else {
-						ext2fs_unmark_valid(fs);
-					}
-				} else {
-				    ext2fs_mark_block_bitmap(block_found_map,
-							     b);
-				    ext2fs_mark_block_bitmap(block_illegal_map,
-							     b);
-				    }
-			}
-		}
-			    
-		/*
-		 * Mark this group's copy of the superblock
-		 */
-		ext2fs_mark_block_bitmap(block_found_map, block);
-		ext2fs_mark_block_bitmap(block_illegal_map, block);
+
+		if (ext2fs_bg_has_super(fs, i)) {
+			/*
+			 * Mark this group's copy of the superblock
+			 */
+			ext2fs_mark_block_bitmap(block_found_map, block);
+			ext2fs_mark_block_bitmap(block_illegal_map, block);
 		
-		/*
-		 * Mark this group's copy of the descriptors
-		 */
-		for (j = 0; j < fs->desc_blocks; j++) {
-			ext2fs_mark_block_bitmap(block_found_map,
-						 block + j + 1);
-			ext2fs_mark_block_bitmap(block_illegal_map,
-						 block + j + 1);
+			/*
+			 * Mark this group's copy of the descriptors
+			 */
+			for (j = 0; j < fs->desc_blocks; j++) {
+				ext2fs_mark_block_bitmap(block_found_map,
+							 block + j + 1);
+				ext2fs_mark_block_bitmap(block_illegal_map,
+							 block + j + 1);
+			}
 		}
 		block += fs->super->s_blocks_per_group;
 	}
@@ -1223,15 +1219,12 @@ errcode_t pass1_get_blocks(ext2_filsys fs, ino_t ino, blk_t *blocks)
 {
 	int	i;
 	
-	if (ino == stashed_ino) {
-		for (i=0; i < EXT2_N_BLOCKS; i++)
-			blocks[i] = stashed_inode->i_block[i];
-		return 0;
-	}
-	printf("INTERNAL ERROR: pass1_get_blocks: unexpected inode #%lu\n",
-	       ino);
-	printf("\t(was expecting %lu)\n", stashed_ino);
-	exit(FSCK_ERROR);
+	if (ino != stashed_ino)
+		return EXT2_ET_CALLBACK_NOTHANDLED;
+
+	for (i=0; i < EXT2_N_BLOCKS; i++)
+		blocks[i] = stashed_inode->i_block[i];
+	return 0;
 }
 
 errcode_t pass1_read_inode(ext2_filsys fs, ino_t ino, struct ext2_inode *inode)
