@@ -15,7 +15,6 @@
  */
 
 #include <stdio.h>
-#include <linux/ext2_fs.h>
 
 #include "e2p.h"
 
@@ -32,6 +31,9 @@ static const unsigned long flags_array[] = {
 #endif
 #ifdef	EXT2_NODUMP_FL
 	EXT2_NODUMP_FL,
+#endif
+#ifdef	EXT2_NOATIME_FL
+	EXT2_NOATIME_FL,
 #endif
 	0};
 
@@ -48,6 +50,9 @@ static const char * short_flags[] = {
 #endif
 #ifdef	EXT2_NODUMP_FL
 	"d",
+#endif
+#ifdef	EXT2_NOATIME_FL
+	"A",
 #endif
 	NULL};
 
@@ -64,6 +69,9 @@ static const char * long_flags[] = {
 #endif
 #ifdef	EXT2_NODUMP_FL
 	"No_Dump, ",
+#endif
+#ifdef	EXT2_NOATIME_FL
+	"No_Atime, ",
 #endif
 	NULL};
 
