@@ -3,8 +3,11 @@
  */
 
 #include <stdio.h>
+#include <string.h>
 #include <linux/types.h>
 
+#include "e2p.h"
+
 struct uuid {
 	__u32	time_low;
 	__u16	time_mid;
