@@ -35,10 +35,15 @@ errcode_t ext2fs_allocate_tables(ext2_filsys fs)
 	group_blk = fs->super->s_first_data_block;
 	for (i = 0; i < fs->group_desc_count; i++) {
 		last_blk = group_blk + fs->super->s_blocks_per_group;
+		if (last_blk >= fs->super->s_blocks_count)
+			last_blk = fs->super->s_blocks_count - 1;
+
 		/*
 		 * Allocate the inode table
 		 */
 		start_blk = group_blk + 3 + fs->desc_blocks;
+		if (start_blk > last_blk)
+			start_blk = group_blk;
 		retval = ext2fs_get_free_blocks(fs, start_blk, last_blk,
 						fs->inode_blocks_per_group,
 						fs->block_map, &new_blk);
@@ -53,8 +58,15 @@ errcode_t ext2fs_allocate_tables(ext2_filsys fs)
 		/*
 		 * Allocate the block and inode bitmaps
 		 */
-		start_blk += fs->inode_blocks_per_group +
-			((2 * i) % (last_blk - start_blk));
+		if (fs->stride) {
+			start_blk += fs->inode_blocks_per_group;
+			start_blk += ((fs->stride * i) %
+				      (last_blk - start_blk));
+			if (start_blk > last_blk)
+				 /* should never happen */
+				start_blk = group_blk;
+		} else
+			start_blk = group_blk;
 		retval = ext2fs_get_free_blocks(fs, start_blk, last_blk,
 						1, fs->block_map, &new_blk);
 		if (retval)
