@@ -2,10 +2,12 @@
  * bitops.c --- Bitmap frobbing code.  See bitops.h for the inlined
  * 	routines.
  * 
- * Copyright (C) 1993, 1994 Theodore Ts'o.  This file may be
- * redistributed under the terms of the GNU Public License.
- * 
- * Taken from <asm/bitops.h>, Copyright 1992, Linus Torvalds.
+ * Copyright (C) 1993, 1994, 1995, 1996 Theodore Ts'o.
+ *
+ * %Begin-Header%
+ * This file may be redistributed under the terms of the GNU Public
+ * License.
+ * %End-Header%
  */
 
 #include <stdio.h>
@@ -72,3 +74,13 @@ void ext2fs_warn_bitmap(errcode_t errcode, unsigned long arg,
 		com_err(0, errcode, "#%u", arg);
 }
 
+void ext2fs_warn_bitmap2(ext2fs_generic_bitmap bitmap,
+			    int code, unsigned long arg)
+{
+	if (bitmap->description)
+		com_err(0, bitmap->base_error_code+code,
+			"#%u for %s", arg, bitmap->description);
+	else
+		com_err(0, bitmap->base_error_code + code, "#%u", arg);
+}
+
