@@ -1,8 +1,12 @@
 /*
  * read_bb_file.c --- read a list of bad blocks for a FILE *
  *
- * Copyright (C) 1994 Theodore Ts'o.  This file may be redistributed
- * under the terms of the GNU Public License.
+ * Copyright (C) 1994, 1995 Theodore Ts'o.
+ *
+ * %Begin-Header%
+ * This file may be redistributed under the terms of the GNU Public
+ * License.
+ * %End-Header%
  */
 
 #include <stdio.h>
@@ -22,7 +26,7 @@
  * Reads a list of bad blocks from  a FILE *
  */
 errcode_t ext2fs_read_bb_FILE(ext2_filsys fs, FILE *f, 
-			      badblocks_list *bb_list,
+			      ext2_badblocks_list *bb_list,
 			      void (*invalid)(ext2_filsys fs, blk_t blk))
 {
 	errcode_t	retval;
@@ -33,7 +37,7 @@ errcode_t ext2fs_read_bb_FILE(ext2_filsys fs, FILE *f,
 	EXT2_CHECK_MAGIC(fs, EXT2_ET_MAGIC_EXT2FS_FILSYS);
 
 	if (!*bb_list) {
-		retval = badblocks_list_create(bb_list, 10);
+		retval = ext2fs_badblocks_list_create(bb_list, 10);
 		if (retval)
 			return retval;
 	}
@@ -50,7 +54,7 @@ errcode_t ext2fs_read_bb_FILE(ext2_filsys fs, FILE *f,
 				(invalid)(fs, blockno);
 			continue;
 		}
-		retval = badblocks_list_add(*bb_list, blockno);
+		retval = ext2fs_badblocks_list_add(*bb_list, blockno);
 		if (retval)
 			return retval;
 	}
