@@ -11,9 +11,11 @@
 #include "ss_internal.h"
 
 #ifdef __STDC__
-#define	DECLARE(name) void name(int argc,char **argv, int sci_idx)
+#define	DECLARE(name) void name(int argc,const char * const *argv, \
+				int sci_idx, void *infop)
 #else
-#define	DECLARE(name) void name(argc,argv,sci_idx)int argc,sci_idx;char **argv;
+#define	DECLARE(name) void name(argc,argv,sci_idx,info)int argc,sci_idx;char **argv;void *infop;
+
 #endif
 	
 /*
