@@ -1,7 +1,15 @@
 /*
  * Internal routine for packing UUID's
+ * 
+ * Copyright (C) 1996, 1997 Theodore Ts'o.
+ *
+ * %Begin-Header%
+ * This file may be redistributed under the terms of the GNU Public
+ * License.
+ * %End-Header%
  */
 
+#include <string.h>
 #include "uuidP.h"
 
 void uuid_pack(struct uuid *uu, uuid_t ptr)
