@@ -6,8 +6,12 @@
  *                                 Laboratoire MASI, Institut Blaise Pascal
  *                                 Universite Pierre et Marie Curie (Paris VI)
  *
- * This file can be redistributed under the terms of the GNU General
- * Public License
+ * Copyright 1995, 1996, 1997 by Theodore Ts'o.
+ *
+ * %Begin-Header%
+ * This file may be redistributed under the terms of the GNU Public
+ * License.
+ * %End-Header%
  */
 
 /*
@@ -39,7 +43,7 @@ char * device_name = NULL;
 
 static volatile void usage (void)
 {
-	fprintf (stderr, "usage: %s device\n", program_name);
+	fprintf (stderr, "usage: %s [-bV] device\n", program_name);
 	exit (1);
 }
 
@@ -73,18 +77,27 @@ static void print_free (unsigned long group, char * bitmap,
 static void list_desc (ext2_filsys fs)
 {
 	unsigned long i;
+	blk_t	group_blk, next_blk;
 	char * block_bitmap = fs->block_map->bitmap;
 	char * inode_bitmap = fs->inode_map->bitmap;
 
 	printf ("\n");
-	for (i = 0; i < fs->group_desc_count; i++)
-	{
-		printf ("Group %lu:\n", i);
-		printf ("  Block bitmap at %u, Inode bitmap at %u, "
-			"Inode table at %u\n",
+	group_blk = fs->super->s_first_data_block;
+	for (i = 0; i < fs->group_desc_count; i++) {
+		next_blk = group_blk + fs->super->s_blocks_per_group;
+		if (next_blk > fs->super->s_blocks_count)
+			next_blk = fs->super->s_blocks_count;
+		printf ("Group %lu: (Blocks %u -- %u)\n", i,
+			group_blk, next_blk -1 );
+		printf ("  Block bitmap at %u (+%d), "
+			"Inode bitmap at %u (+%d)\n  "
+			"Inode table at %u (+%d)\n",
 			fs->group_desc[i].bg_block_bitmap,
+			fs->group_desc[i].bg_block_bitmap - group_blk,
 			fs->group_desc[i].bg_inode_bitmap,
-			fs->group_desc[i].bg_inode_table);
+			fs->group_desc[i].bg_inode_bitmap - group_blk,
+			fs->group_desc[i].bg_inode_table,
+			fs->group_desc[i].bg_inode_table - group_blk);
 		printf ("  %d free blocks, %d free inodes, %d directories\n",
 			fs->group_desc[i].bg_free_blocks_count,
 			fs->group_desc[i].bg_free_inodes_count,
@@ -98,6 +111,7 @@ static void list_desc (ext2_filsys fs)
 		print_free (i, inode_bitmap, fs->super->s_inodes_per_group, 1);
 		inode_bitmap += fs->super->s_inodes_per_group / 8;
 		printf ("\n");
+		group_blk = next_blk;
 	}
 }
 
@@ -150,25 +164,39 @@ static void dump_bad_blocks(ext2_filsys fs)
 	badblocks_list_iterate_end(bb_iter);
 }
 
+static int i386_byteorder(void)
+{
+	int one = 1;
+	char *cp = (char *) &one;
+
+	return (*cp == 1);
+}
 
 void main (int argc, char ** argv)
 {
 	errcode_t	retval;
 	ext2_filsys	fs;
 	int		print_badblocks = 0;
+	int		big_endian;
 	char		c;
 
+	initialize_ext2_error_table();
 	fprintf (stderr, "dumpe2fs %s, %s for EXT2 FS %s, %s\n",
 		 E2FSPROGS_VERSION, E2FSPROGS_DATE,
 		 EXT2FS_VERSION, EXT2FS_DATE);
 	if (argc && *argv)
 		program_name = *argv;
 	
-	while ((c = getopt (argc, argv, "b")) != EOF) {
+	while ((c = getopt (argc, argv, "bV")) != EOF) {
 		switch (c) {
 		case 'b':
 			print_badblocks++;
 			break;
+		case 'V':
+			/* Print version number and exit */
+			fprintf(stderr, "\tUsing %s\n",
+				error_message(EXT2_ET_BASE));
+			exit(0);
 		default:
 			usage ();
 		}
@@ -176,7 +204,6 @@ void main (int argc, char ** argv)
 	if (optind > argc - 1)
 		usage ();
 	device_name = argv[optind++];
-	initialize_ext2_error_table();
 	retval = ext2fs_open (device_name, 0, 0, 0, unix_io_manager, &fs);
 	if (retval) {
 		com_err (program_name, retval, "while trying to open %s",
@@ -195,6 +222,11 @@ void main (int argc, char ** argv)
 			ext2fs_close (fs);
 			exit (1);
 		}
+		big_endian = ((fs->flags & EXT2_FLAG_SWAP_BYTES) != 0);
+		if (!i386_byteorder())
+			big_endian = !big_endian;
+		if (big_endian)
+			printf("Note: This is a byte-swapped filesystem\n");
 		list_super (fs->super);
 		list_bad_blocks (fs);
 		list_desc (fs);
